#ifndef _LINUX_LIST_H
#define _LINUX_LIST_H


struct list_head {
         struct list_head *next, *prev;
};

#define LIST_HEAD_INIT(name) { &(name), &(name) }



#define LIST_HEAD(name) \
	struct list_head name = LIST_HEAD_INIT(name)



#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)

#define container_of(ptr, type, member) ({                      \
        const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
        (type *)( (char *)__mptr - offsetof(type,member) );})

#define __container_of(ptr, sample, member)				\
    (void *)container_of((ptr), typeof(*(sample)), member)


static inline void INIT_LIST_HEAD(struct list_head *list);

static inline void __list_add(struct list_head *pnew, struct list_head *prev,struct list_head *next);

static inline void list_add(struct list_head *pnew, struct list_head *head);

static inline void __list_del(struct list_head * prev, struct list_head * next);

static inline void list_del(struct list_head *entry);


#define prefetch(x) __builtin_prefetch(x)



#define list_for_each(pos, head) \
         for (pos = (head)->next; prefetch(pos->next), pos != (head); \
                 pos = pos->next)

#define list_entry(ptr, type, member) \
         container_of(ptr, type, member)

#define list_for_each_entry(pos, head, member)				\
    for (pos = __container_of((head)->next, pos, member);		\
	 &pos->member != (head);					\
	 pos = __container_of(pos->member.next, pos, member))

#define list_for_each_entry_safe(pos, n, head, member)			\
	for (pos = list_entry((head)->next, typeof(*pos), member),	\
		n = list_entry(pos->member.next, typeof(*pos), member);	\
	     &pos->member != (head);					\
	     pos = n, n = list_entry(n->member.next, typeof(*n), member))

#endif
