#ifndef VBIOS_H
#define VBIOS_H
#include <QMainWindow>
#include <QLinkedList>
#include <QtGlobal>
#include <QMap>
#include "loongson_data.h"

typedef quint8 u8;
typedef quint16 u16;
typedef quint32 u32;
typedef quint64 u64;

typedef qint8 s8;
typedef qint16 s16;
typedef qint32 s32;
typedef qint64 s64;
typedef bool (*parse_func)(struct desc_node *, struct vbios_cmd *);

struct desc_func {
    enum desc_type type;
    u16 ver;
    s8 *name;
    u8 index;
    parse_func func;
} __attribute__((packed));

struct vbios_i2c {
       u32 feature;
       u16  id;
       enum i2c_type type;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_i2c)

struct vbios_header {
    u32 feature;
    u8  oem_vendor[32];
    u8  oem_product[32];
    u32 legacy_offset;
    u32 legacy_size;
    u32 desc_offset;
    u32 desc_size;
    u32 data_offset;
    u32 data_size;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_header)

struct vbios_pwm {
    u32 feature;
    u8  pwm;
    u8  polarity;
    u32 peroid;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_pwm)

struct vbios_backlight {
    u32 feature;
    u8  used;
    enum vbios_backlight_type  type;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_backlight)

struct vbios_desc {
    u16 type; //
    u8  ver;
    u8  index;
    u32 offset;
    u32 size;
    u64 ext[2];
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_desc)

struct vbios_cmd {
    u8    index;
    enum desc_type type;
    void *req;
    void *res;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_cmd)

struct loongson_vbios {
    char title[16];
    uint32_t version_major;
    uint32_t version_minor;
    char information[20];
    uint32_t crtc_num;
    uint32_t crtc_offset;
    uint32_t connector_num;
    uint32_t connector_offset;
    uint32_t encoder_num;
    uint32_t encoder_offset;
}__attribute__ ((packed));
Q_DECLARE_METATYPE(loongson_vbios)

struct vbios_crtc {
    u32 feature;
    u32 crtc_id; // HW_id
    u32 encoder_id; // encoder_index
    u32 max_freq;
    u32 max_width;
    u32 max_height;
    bool is_vb_timing;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_crtc)


struct vbios_encoder {
    u32 feature;
    u32 i2c_id; // i2c->data->id
    u32 connector_id; // conn_index
    enum encoder_type type;
    enum encoder_config config_type;
    enum encoder_object chip;
    u8 chip_addr;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_encoder)

struct vbios_connector {
    u32 feature;
    u32 i2c_id; //i2c->data->id
    u8 internal_edid[256];
    enum connector_type type;
    enum hotplug hotplug;
    enum loongson_edid_method edid_method;
    u32 irq_gpio;
    enum gpio_placement irq_placement;
} __attribute__((packed));
Q_DECLARE_METATYPE(vbios_connector)

struct vbios_cfg_encoder {
    u32 hdisplay;
    u32 vdisplay;
    u8 reg_num;
    struct vbios_conf_reg config_regs[256];
} __attribute__((packed));

struct list_head{
    struct list_head *next,*prev;
};

struct loongson_device {
    struct list_head desc_list;
    void *vbios;
};
Q_DECLARE_METATYPE(loongson_device)

struct block_list {
    struct vbios_desc *desc;
    void *data;
};
Q_DECLARE_METATYPE(block_list)

typedef struct desc_fuc {
    enum desc_type type;
    u16 ver;
    parse_func func;
    s8 *name;
    u8 index;
}desc_fuc_t;

struct desc_node {
    struct list_head head;
    u8 *data;
    struct vbios_desc *desc;
    parse_func parse;
} __attribute__((packed));

struct all_data{
    struct vbios_crtc  *crtc;
    struct vbios_encoder *encoder;
    struct vbios_connector *connector;
    struct vbios_pwm *pwm;
    int pwm_index = 0;
    struct vbios_i2c *i2c;
    struct vbios_backlight *bl;
    int bl_index = 0;
    struct vbios_cfg_encoder *cfg[10];
} __attribute__((packed));

struct edid_conn{
    int number;
    u8 internal_edid[256];
};

struct config_encoder{
    u8 reg_num;
    u32 hdisplay;
    u32 vdisplay;
    struct vbios_conf_reg config_regs[256];
};

struct cfg_info{
    int num_group;
    struct config_encoder tmp_cfg[10];
};

#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)

#define container_of(ptr, type, member) ({                      \
        const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
        (type *)( (char *)__mptr - offsetof(type,member) );})

#define __container_of(ptr, sample, member)				\
    (void *)container_of((ptr), typeof(*(sample)), member)


#define list_for_each(pos, head) \
         for (pos = (head)->next; prefetch(pos->next), pos != (head); \
                 pos = pos->next)

#define list_entry(ptr, type, member) \
         container_of(ptr, type, member)

#define list_for_each_entry(pos, head, member)				\
    for (pos = __container_of((head)->next, pos, member);		\
     &pos->member != (head);					\
     pos = __container_of(pos->member.next, pos, member))

#define list_for_each_entry_safe(pos, n, head, member)			\
    for (pos = list_entry((head)->next, typeof(*pos), member),	\
        n = list_entry(pos->member.next, typeof(*pos), member);	\
         &pos->member != (head);					\
         pos = n, n = list_entry(n->member.next, typeof(*n), member))
#endif // VBIOS_H

