#include "widget.h"
#include "ui_widget.h"
#include "vbios.h"
#include <QFile>
#include <QFileDialog>
#include <QPalette>
#include <QPixmap>
#include <QIcon>
#include <QMovie>
#include <QDebug>
#include <QStandardPaths>
#include <QFileInfo>
#include <QDesktopServices>
#include <QTextCodec>
#include <QTextStream>
#include <QFile>
#include <QDesktopServices>


static int num = 1;
static FILE *fp =nullptr;
static int parse_flag = -1;
static all_data *parse_all = nullptr;
static int dvo_flag = -1;
static int num_link = 0;


static char *desc_name[]={
    "header",
    "crtc",
    "encoder",
    "connector",
    "i2c",
    "pwm",
    "gpio",
    "backlight",
    "fan",
    "irq_vblank",
    "cfg_encoder",
    "max"
};

#define FUNC(t, v, f) \
    { \
        .type = t, .ver = v, .func = f,\
    }

desc_fuc_t tables[] = {
    FUNC(desc_backlight, ver_v1, parse_vbios_backlight),
    FUNC(desc_pwm, ver_v1, parse_vbios_pwm),
    FUNC(desc_header, ver_v1, parse_vbios_header),
    FUNC(desc_crtc, ver_v1, parse_vbios_crtc),
    FUNC(desc_connector, ver_v1, parse_vbios_connector),
    FUNC(desc_encoder, ver_v1, parse_vbios_encoder),
    FUNC(desc_cfg_encoder, ver_v1, parse_vbios_cfg_encoder),
    FUNC(desc_i2c, ver_v1, parse_vbios_i2c),
};

Widget::Widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Widget), mem_node(new loongson_device)
{
    ui->setupUi(this);
    ui->tabWidget->setTabText(0,"link_1");
    ui->tabWidget->setTabText(1,"link_2");
    all = new all_data[2];

    if(nullptr == all){
        qDebug("all allocate memory failed!\n");
    }
    memset(all,0,2*sizeof(struct all_data));
    for(int i=0;i<2;i++){
        all[i].crtc = new vbios_crtc();
        memset(all[i].crtc,0,sizeof(struct vbios_crtc));

        all[i].encoder = new vbios_encoder();
        memset(all[i].encoder,0,sizeof(struct vbios_encoder));

        all[i].connector = new vbios_connector();
        memset( all[i].connector,0,sizeof(struct vbios_connector));

        all[i].pwm = new vbios_pwm();
        memset(all[i].pwm,0,sizeof(struct vbios_pwm));

        all[i].bl = new vbios_backlight();
        memset(all[i].bl,0,sizeof(struct vbios_backlight));

        all[i].i2c = new vbios_i2c();
        memset(all[i].i2c,0,sizeof(struct vbios_i2c));

         for(int j=0;j<10;j++){
            all[i].cfg[j] = new vbios_cfg_encoder();
            memset(all[i].cfg[j],0,sizeof(vbios_cfg_encoder));
        }
    }

    allocate_parse_memory();

    mem_node->vbios = (u8 *)calloc(1,1024*256);
    memset(mem_node->vbios,0x00,sizeof(u8)*256*1024);

    ldev.append(*mem_node);
    ldevit = ldev.begin();


    config_page1 = new encoder_config_page1();
    config_page2 = new encoder_config_page2();

    edid_page1 = new connector_edid_page1();
    edid_page2 = new connector_edid_page2();


    hexspin1 = new HexSpinBox(this);
    QWidget *w1 = ui->tabWidget->widget(0);
    hexspin1->setParent(w1);
    hexspin1->resize(50,35);
    hexspin1->setGeometry(QRect(160,230,60,35));
    QFont font = hexspin1->font();
    font.setCapitalization(QFont::AllLowercase);
    hexspin1->setFont(font);
    hexspin1->setPrefix("0x");
    hexspin1->setInputMethodHints(Qt::ImhPreferUppercase);
    hexspin1->show();
    connect(hexspin1,SIGNAL(editingFinished()),this,SLOT(phy_edit_data()));

    hexspin2 = new HexSpinBox(this);
    hexspin2->setGeometry(QRect(160,240,60,35));
    QWidget *w2= ui->tabWidget->widget(1);
    hexspin2->setPrefix("0x");
    hexspin2->setInputMethodHints(Qt::ImhPreferUppercase);
    hexspin2->setParent(w2);
    hexspin2->show();
    connect(hexspin2,SIGNAL(editingFinished()),this,SLOT(phy_edit_data2()));

    ui->tabWidget->setTabEnabled(0,true);
    ui->tabWidget->setTabEnabled(1,false);

    this->setWindowTitle("loongson-vbios-creator-1.0");
    this->Set_shortcut_icon();
    this->dev_properlty_init();

    this->setFixedSize(930,760);
}

Widget::~Widget()
{
    delete [] all->crtc;
    delete [] all->encoder;
    delete [] all->connector;
    delete [] all->pwm;
    delete [] all->bl;
    for(int i=0;i<10;i++){
        delete [] all[1].cfg[i];
        delete [] all[0].cfg[i];
    }

    delete [] all;
    this->free_parse_memory();

    delete ui;
    delete edid_page1;
    delete edid_page2;

    delete config_page1;
    delete config_page2;
}

void Widget::phy_edit_data()
{
    bool ok;
    int data = 0;
    QString temp = hexspin1->text();
    data = temp.toInt(&ok,16);
    all[0].encoder->chip_addr = data;
    return;
}

void Widget::phy_edit_data2()
{
    bool ok;
    int data = 0;
    QString temp = hexspin2->text();
    data = temp.toInt(&ok,16);
    all[1].encoder->chip_addr = data;
    return;
}

void Widget::Set_shortcut_icon()
{
    ui->save_btn->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    ui->save_btn->setIcon(QIcon(":/icon/icon/save.png"));
    ui->save_btn->resize(80,50);

    ui->check_btn->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    ui->check_btn->setIcon(QIcon(":/icon/icon/check.png"));
    ui->check_btn->resize(80,50);

    ui->load_btn->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    ui->load_btn->setIcon(QIcon(":/icon/icon/load.png"));
    ui->load_btn->resize(80,50);

    ui->faq_btn->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    ui->faq_btn->setIcon(QIcon(":/icon/icon/faq.png"));
    ui->faq_btn->resize(80,50);

    ui->exit_btn->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    ui->exit_btn->setIcon(QIcon(":/icon/icon/exit.png"));
    ui->exit_btn->resize(80,50);

    ui->document_btn->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    ui->document_btn->setIcon(QIcon(":/icon/icon/Document.png"));
    ui->document_btn->resize(80,50);

    QMovie *movie=new QMovie("://new/prefix1/res/logo.png");
    ui->label_4->resize(100,100);
    ui->label_4->setAlignment(Qt::AlignCenter);
    ui->label_4->setMovie(movie);

    QPixmap pixmap("://new/prefix1/res/logo.png");
    ui->label_4->resize(400,150);
    ui->label_4->setPixmap(pixmap);
    ui->label_4->show();
    return;
}

void Widget::dev_properlty_init()
{
    ui->crtc_id_spin->setValue(0);
    ui->crtc_id_spin_2->setValue(0);

    ui->crtc_freq_spin->setRange(0,2147483640);
    ui->crtc_freq_spin->setValue(200000);

    ui->freq_spin_2->setRange(0,2147483640);
    ui->freq_spin_2->setValue(200000);

    ui->encoder_type_combo->setCurrentIndex(0);
    ui->encoder_type_combo_2->setCurrentIndex(0);

    ui->config_type_combo->setCurrentIndex(1);
    ui->config_type_combo_2->setCurrentIndex(1);

    ui->chip_combo->setCurrentIndex(0);
    ui->chip_combo_2->setCurrentIndex(0);

    ui->connect_type_combo->setCurrentIndex(0);
    ui->connect_type_combo_2->setCurrentIndex(0);

    ui->irq_gpio_spin->setValue(0);
    ui->irq_gpio_spin->hide();
    ui->irq_gpio_spin_2->setValue(0);
    ui->irq_gpio_spin_2->hide();

    ui->irq_placement_combo->setCurrentIndex(0);
    ui->irq_placement_combo->hide();
    ui->irq_placement_combo_2->setCurrentIndex(0);
    ui->irq_placement_combo_2->hide();

    ui->irq_gpio_lb->hide();
    ui->irq_gpio_lab2->hide();
    ui->irq_placement_la->hide();
    ui->irq_plament_lab2->hide();

    ui->hotplug_combo->setCurrentIndex(0);
    ui->hotplug_combo_2->setCurrentIndex(0);

    ui->edid_method_combo->setCurrentIndex(0);
    ui->connector_edid->hide();

    ui->edid_method_combo_2->setCurrentIndex(0);
    ui->connector_edid_2->hide();

    ui->pwm_id_spin->setValue(0);
    ui->pwm_id_spin_2->setValue(0);

    ui->polarity_spin->setValue(0);
    ui->polarity_spin_2->setValue(0);

    ui->peroid_spin->setRange(0,2147483640);
    ui->peroid_spin->setValue(100000);

    ui->peroid_spin_2->setRange(0,2147483640);
    ui->peroid_spin_2->setValue(100000);

    ui->bl_type_combo->setCurrentIndex(0);
    ui->bl_type_combo2->setCurrentIndex(0);

    ui->bl_used_spin->setValue(0);
    ui->bl_used_spin2->setValue(0);

    ui->i2c_type_combo->setCurrentIndex(0);
    ui->i2c_type_combo_2->setCurrentIndex(0);
    return;
}

void Widget::on_tabWidget_currentChanged(int index)
{
    ui->tabWidget->setCurrentIndex(index);
}

void Widget::on_crtc_id_spin_valueChanged(int arg1)
{
    all[0].crtc->crtc_id = arg1;
}

void Widget::on_crtc_freq_spin_valueChanged(int arg1)
{
    all[0].crtc->max_freq = arg1;
}


void Widget::on_encoder_type_combo_currentIndexChanged(int index)
{
    all[0].encoder->type = (enum encoder_type)index;
}

void Widget::on_config_type_combo_currentIndexChanged(int index)
{
    if(index == 0)
        all[0].encoder->config_type = encoder_os_config;
    else if(index == 1)
        all[0].encoder->config_type = encoder_timing_filling;
    else if(index == 2)
        all[0].encoder->config_type = encoder_kernel_driver;

    if(all[0].encoder->config_type == encoder_timing_filling || all[0].encoder->config_type == encoder_kernel_driver )
        ui->encoder_config_btn->hide();
    else
        ui->encoder_config_btn->show();
}

void Widget::on_chip_combo_currentIndexChanged(int index)
{
    all[0].encoder->chip = (enum encoder_object)index;
}

void Widget::on_connect_type_combo_currentIndexChanged(int index)
{
    all[0].connector->type = (enum connector_type)index;
}


void Widget::on_hotplug_combo_currentIndexChanged(int index)
{
    all[0].connector->hotplug = (enum hotplug)index;
    if(all[0].connector->hotplug == irq){
        ui->irq_placement_la->show();
        ui->irq_placement_combo->show();

        ui->irq_gpio_lb->show();
        ui->irq_gpio_spin->show();
    }else{
        ui->irq_placement_la->hide();
        ui->irq_placement_combo->hide();

        ui->irq_gpio_lb->hide();
        ui->irq_gpio_spin->hide();
    }
}


void Widget::on_pwm_id_spin_valueChanged(int arg1)
{
    all[0].pwm->pwm = arg1;
}

void Widget::on_polarity_spin_valueChanged(int arg1)
{
    all[0].pwm->polarity = arg1;
}

void Widget::on_peroid_spin_valueChanged(int arg1)
{
    all[0].pwm->peroid = arg1;
}


void Widget::make_table()
{
    int i, j ;
    have_table = 1 ;
    for (i = 0 ; i < 256 ; i++)
        for (j = 0, table[i] = i ; j < 8 ; j++)
            table[i] = (table[i]>>1)^((table[i]&1)?POLYNOMIAL:0) ;
    return;
}

uint Widget::crc32(uint crc, char *buff, int len)
{
    if (!have_table) this->make_table();
    crc = ~crc;
    for (int i = 0; i < len; i++)
        crc = (crc >> 8) ^ table[(crc ^ buff[i]) & 0xff];
    return ~crc;
}

u32 Widget::get_desc_node_num(void)
{
    u32 dnum=0;
    dnum = vbt_block_list.count();
    return dnum+1;
}

u32 Widget::get_data_size(void)
{
    u32 size=0;
    struct vbios_desc *desc;
    for(bliterator=vbt_block_list.begin();bliterator!=vbt_block_list.end();bliterator++)
    {
        desc = bliterator->desc;
        size += desc->size;
    }
    return size;
}

void *Widget::get_data_by_type(enum desc_type type)
{
    struct vbios_desc *desc;
    for(bliterator=vbt_block_list.begin();bliterator!=vbt_block_list.end();bliterator++)
    {
       desc = bliterator->desc;
       if(desc->type == type)
           return bliterator->data;
    }
    return nullptr;
}

int Widget::fill_vbios_header(void)
{
    u32 desc_num;
    u32 desc_size;
    u32 data_offset;
    u32 data_size;
    u32 total_msg;
    struct vbios_desc *desc;
    struct vbios_header *header;

    desc_num = this->get_desc_node_num();
    desc_size = desc_num * sizeof(struct vbios_desc);
    data_size = this->get_data_size();
    data_offset = VBIOS_DESC_OFFSET + desc_size;

    header = (struct vbios_header*)this->get_data_by_type(desc_header);
    if (!header) {
            return 1;
    }

    header->desc_size = desc_size;
    header->desc_offset = VBIOS_DESC_OFFSET;
    header->data_size = data_size;
    header->data_offset = VBIOS_DESC_OFFSET + header->desc_size;
    total_msg = header->data_size + header->desc_size + VBIOS_DESC_OFFSET;

    if( total_msg > VBIOS_SIZE ){
     return 1;
    }
    return 0;
}

int Widget::simu_vbios_header(int num_index)
{
    struct vbios_header *header;
    struct vbios_desc *desc;
    struct block_list *node = (struct block_list *)malloc(sizeof(*node));
    if (!node) {
       qDebug("alloc node mm err\n");
       return 1;
    }

    header = (struct vbios_header *)malloc(sizeof(*header));

    if(!header){
        free(node);
        node = nullptr;
        return 1;
    }
    desc = (struct vbios_desc *)malloc(sizeof(*desc));
    if(!desc){
        goto err1;
    }
    sprintf((char*)header->oem_vendor, "header-%d", num_index);
    sprintf((char*)header->oem_product, "2020");
    desc->index = num_index;
    desc->size = sizeof(*header);
    desc->type = desc_header;
    desc->ver = ver_v1;

    node->data = (void *)header;
    node->desc = desc;
    vbt_block_list.append(*node);
    return 0;
err1:
    free(header);
    free(node);
    header = nullptr;
    node = nullptr;
    return 1;
}

int Widget::simu_vbios_crtc()
{
    struct vbios_crtc *vbt_crtc = nullptr;
    struct vbios_desc *desc = nullptr;
    for (int crtc_num=0; crtc_num < 2; crtc_num++){
        if(!all[crtc_num].encoder->type)
            continue;
    struct block_list *node = (struct block_list *)malloc(sizeof(*node));
    if (!node) {
        return 1;
    }
    vbt_crtc = (struct vbios_crtc *)malloc(sizeof(*vbt_crtc));

    if(!vbt_crtc){
       free(node);
       node = nullptr;
       return 1;
    }

    desc = (struct vbios_desc *)malloc(sizeof(*desc));
    if(!desc){
       free(vbt_crtc);
       vbt_crtc = nullptr;
       free(node);
       node = nullptr;
       return 1;
    }
        vbt_crtc->feature = 0;
        vbt_crtc->crtc_id = all[crtc_num].crtc->crtc_id;
        vbt_crtc->encoder_id = crtc_num;
        vbt_crtc->max_freq = all[crtc_num].crtc->max_freq;
        vbt_crtc->max_height = 2080;
        vbt_crtc->max_width = 2080;
        vbt_crtc->is_vb_timing = 0;

        desc->index = crtc_num;
        desc->size = sizeof(*vbt_crtc);
        desc->type = desc_crtc;
        desc->ver = ver_v1;

        node->data = (void *)vbt_crtc;
        node->desc = desc;
        vbt_block_list.append(*node);
  }
  return 0;
}


int Widget::simu_vbios_i2c()
{
    struct vbios_i2c *vbt_i2c, *tmp = nullptr;
    struct vbios_desc *desc = nullptr;
    struct block_list *node = nullptr;

    tmp = vbt_i2c =(struct vbios_i2c *) malloc(sizeof(*vbt_i2c)* num);
    if (nullptr == vbt_i2c)
           return 1;

    for (int i2c_num = 0; i2c_num < num; i2c_num++){
        vbt_i2c->feature = 1;
        vbt_i2c->id = i2c_num;
        vbt_i2c->type = all[i2c_num].i2c->type;
        vbt_i2c++;
    }

    desc = (struct vbios_desc *)malloc(sizeof(*desc));
    if (nullptr == desc) {
        free(vbt_i2c);
        vbt_i2c = nullptr;
        return 1;
    }

    desc->index = 0;
    desc->size = sizeof(*vbt_i2c)*num;
    desc->type = desc_i2c;
    desc->ver = ver_v1;
    node = (struct block_list *)malloc(sizeof(*node));

    if (!node) {
        free(desc);
        free(vbt_i2c);
        return 1;
    }

    node->data = (void *)tmp;
    node->desc = desc;
    vbt_block_list.append(*node);
    return 0;
}


int Widget::simu_vbios_encoder()
{
    struct vbios_encoder *vbt_encoder = nullptr;
    struct vbios_desc *desc = nullptr;
    for (int enc_num = 0; enc_num < 2; enc_num++)
    {
       if(!all[enc_num].encoder->type){
           continue;
       }

       struct block_list *node = (struct block_list *)malloc(sizeof(*node));
       if (!node) {
           return 1;
       }

       vbt_encoder = (struct vbios_encoder *)malloc(sizeof(*vbt_encoder));
       desc = (struct vbios_desc *)malloc(sizeof(*desc));
       if(!desc){
          free(vbt_encoder);
          vbt_encoder = nullptr;
          free(node);
          node = nullptr;
          return 1;
       }
       vbt_encoder->feature = 0;
       vbt_encoder->i2c_id = enc_num;
       vbt_encoder->connector_id = enc_num;
       vbt_encoder->type = all[enc_num].encoder->type;
       vbt_encoder->config_type = all[enc_num].encoder->config_type;
       vbt_encoder->chip_addr = all[enc_num].encoder->chip_addr;


       switch (all[enc_num].encoder->chip) {
           case 0:
               vbt_encoder->chip = Unknown;
               break;
           case 1:
               vbt_encoder->chip = Internal_Dvo;
               break;
           case 2:
               vbt_encoder->chip = Internal_Hdmi;
               break;
           case 3:
               vbt_encoder->chip = Vga_Ch7055;
               break;
           case 4:
               vbt_encoder->chip = Vga_Adv7125;
               break;
           case 5:
               vbt_encoder->chip = Dvi_Tfp410;
               break;
           case 6:
               vbt_encoder->chip = Hdmi_It66121;
               break;
           case 7:
               vbt_encoder->chip = Hdmi_Sil9022;
               break;
           case 8:
               vbt_encoder->chip = Hdmi_Lt8618;
               break;
           case 9:
               vbt_encoder->chip = Hdmi_Ms7210;
               break;
           case 10:
               vbt_encoder->chip = Edp_Ncs8805;
               break;
           default:
               break;
       }

       desc->index = enc_num;
       desc->size = sizeof(*vbt_encoder);
       desc->type = desc_encoder;
       desc->ver = ver_v1;

       node->data = (void *)vbt_encoder;
       node->desc = desc;
       vbt_block_list.append(*node);
   }

   return 0;
}

int Widget::simu_vbios_connector()
{
    struct vbios_connector *vbt_conn = nullptr;
    struct vbios_desc *desc = nullptr;
    u32 gpio = -1;
    for (int conn_num = 0; conn_num < 2; conn_num++){
         if(!all[conn_num].connector->type){
             continue;
         }
         struct block_list *node = (struct block_list *)malloc(sizeof(*node));
         if (!node) {
              return 1;
         }

         vbt_conn = (struct vbios_connector *)malloc(sizeof(*vbt_conn));
         if (!vbt_conn) {
            free(node);
            node = nullptr;
            return 1;
         }

         desc = (struct vbios_desc *)malloc(sizeof(*desc));
         if(!desc){
            free(vbt_conn);
            vbt_conn = nullptr;
            free(node);
            node = nullptr;
            return 1;
        }

        vbt_conn->feature = 0;
        vbt_conn->i2c_id = conn_num;
        vbt_conn->type = all[conn_num].connector->type;
        vbt_conn->hotplug = all[conn_num].connector->hotplug;
        if(!all[conn_num].connector->edid_method)
            vbt_conn->edid_method = via_i2c;
         else if(all[conn_num].connector->edid_method)
            vbt_conn->edid_method = all[conn_num].connector->edid_method;

        vbt_conn->irq_gpio = gpio;

        if(all[conn_num].connector->hotplug == irq){
             vbt_conn->irq_gpio = all[conn_num].connector->irq_gpio;
             vbt_conn->irq_placement = all[conn_num].connector->irq_placement;
        }

        if(all[conn_num].connector->edid_method == via_vbios && conn_num == 0){
            for(int k=0;k<edid_page1->EDID_INFO->number;k++){
                vbt_conn->internal_edid[k] = edid_page1->EDID_INFO->internal_edid[k];
            }
        } else if(all[conn_num].connector->edid_method == via_vbios && conn_num == 1){
            for(int p=0;p<edid_page2->EDID_INFO2->number;p++){
                vbt_conn->internal_edid[p] = edid_page2->EDID_INFO2->internal_edid[p];
            }
        }

        desc->index = conn_num;
        desc->size = sizeof(*vbt_conn);
        desc->type = desc_connector;
        desc->ver = ver_v1;

        node->data = (void *)vbt_conn;
        node->desc = desc;
        vbt_block_list.append(*node);
   }
   return 0;
}

int Widget::simu_vbios_pwm()
{
    struct vbios_pwm *vbt_pwm = nullptr;
    struct vbios_desc *desc = nullptr;
    for (int pwm_num = 0; pwm_num < 2; pwm_num++){
         if(!all[pwm_num].pwm->pwm){
             continue;
          }
          struct block_list *node = (struct block_list *)malloc(sizeof(*node));
          if (!node) {
               return 1;
          }

          vbt_pwm = (struct vbios_pwm *)malloc(sizeof(*vbt_pwm));
          if(!vbt_pwm){
              free(node);
          return 1;
          }
          desc = (struct vbios_desc *)malloc(sizeof(*desc));
          if(!desc){
             free(vbt_pwm);
         vbt_pwm = nullptr;
         free(node);
         node = nullptr;
         return 1;
       }
      vbt_pwm->feature = 1;
      vbt_pwm->peroid = all[pwm_num].pwm->peroid;
      vbt_pwm->polarity = all[pwm_num].pwm->polarity;
      vbt_pwm->pwm = all[pwm_num].pwm->pwm;

      desc->index = pwm_num;
      desc->size = sizeof(*vbt_pwm);
      desc->type = desc_pwm;
      desc->ver = ver_v1;

      node->data = (void *)vbt_pwm;
      node->desc = desc;
      vbt_block_list.append(*node);
    }
    return 0;
}

int Widget::simu_vbios_backlight()
{
    struct vbios_backlight *vbt_bl = NULL;
    struct vbios_desc *desc = NULL;
    for (int bl_num = 0; bl_num < 2; bl_num++){
        if(!all[bl_num].bl->type)
            continue;
        struct block_list *node = (struct block_list *)malloc(sizeof(*node));
        if (!node) {
          return 1;
        }

        vbt_bl = (struct vbios_backlight *)malloc(sizeof(*vbt_bl));
        desc = (struct vbios_desc *)malloc(sizeof(*desc));

        if (!vbt_bl || !desc) {
              printf("%s malloc sub mm err \n", __func__);
              return 0;
        }

        vbt_bl->feature = 1;
        vbt_bl->type = all[bl_num].bl->type;
        vbt_bl->used = all[bl_num].bl->used;

        desc->index = bl_num;
        desc->size = sizeof(*vbt_bl);
        desc->type = desc_backlight;
        desc->ver = ver_v1;

        node->data = (void *)vbt_bl;
        node->desc = desc;
        vbt_block_list.append(*node);
    }
    return 0;
}

int Widget::simu_vbios_cfg_encoder()
{
    struct vbios_cfg_encoder *vbt_cfg_en,*tmp= nullptr;
    struct vbios_desc *desc = nullptr;

    int cfg_group1 = config_page1->statistics_config_group1();
    int cfg_group2 = config_page2->statistics_config_group();

    int cfg_arr[2]={0};
    cfg_arr[0] = cfg_group1;
    cfg_arr[1] = cfg_group2;

    if(!cfg_group1 && !cfg_group2)
        return 1;

    for(int i=0;i<2;i++){
        struct block_list *node = (struct block_list *)malloc(sizeof(*node));
        if(!node)
            return 1;

        tmp=vbt_cfg_en = (struct vbios_cfg_encoder *)malloc(sizeof(*vbt_cfg_en)*cfg_arr[i]);
        desc = (struct vbios_desc *)malloc(sizeof(*desc));
        if (!vbt_cfg_en || !desc)
            return 1;

         for(int k=0;k<cfg_arr[i];k++){
             if(all[i].encoder->config_type == encoder_os_config){
                if(i==0 && cfg_arr[0]){
                        vbt_cfg_en->reg_num  =  config_page1->all_cfg1[k].reg_num;
                        vbt_cfg_en->hdisplay =  config_page1->all_cfg1[k].hdisplay;
                        vbt_cfg_en->vdisplay =  config_page1->all_cfg1[k].vdisplay;

                        for(int m=0;m<config_page1->all_cfg1[k].reg_num;m++){
                            vbt_cfg_en->config_regs[m].dev_addr = config_page1->all_cfg1[k].config_regs[m].dev_addr;
                            vbt_cfg_en->config_regs[m].reg = config_page1->all_cfg1[k].config_regs[m].reg;
                            vbt_cfg_en->config_regs[m].value = config_page1->all_cfg1[k].config_regs[m].value;
                        }
                }else if(i==1 && cfg_arr[1]){
                        vbt_cfg_en->reg_num  =  config_page2->all_cfg2[k].reg_num;
                        vbt_cfg_en->hdisplay =  config_page2->all_cfg2[k].hdisplay;
                        vbt_cfg_en->vdisplay =  config_page2->all_cfg2[k].vdisplay;

                        for(int m=0;m<config_page2->all_cfg2[k].reg_num;m++){
                            vbt_cfg_en->config_regs[m].dev_addr = config_page2->all_cfg2[k].config_regs[m].dev_addr;
                            vbt_cfg_en->config_regs[m].reg = config_page2->all_cfg2[k].config_regs[m].reg;
                            vbt_cfg_en->config_regs[m].value = config_page2->all_cfg2[k].config_regs[m].value;
                        }
                    }
                }
                    vbt_cfg_en++;
             }
            desc->index = i;
            desc->size = sizeof(*vbt_cfg_en)*cfg_arr[i];
            desc->type = desc_cfg_encoder;
            desc->ver = ver_v1;

            node->data = (void *)tmp;
            node->desc = desc;
            vbt_block_list.append(*node);
        }
    return 0;
}


int Widget::loongson_vbios_simu_data(s32 num_index)
{
    this->simu_vbios_header(num_index);
    this->simu_vbios_crtc();
    this->simu_vbios_encoder();
    this->simu_vbios_connector();
    this->simu_vbios_i2c();
    this->simu_vbios_pwm();
    this->simu_vbios_backlight();
    this->simu_vbios_cfg_encoder();
    this->vbios_assembly_vbios();
    return 0;
}

int Widget::fill_vbios_desc(void)
{
    struct vbios_header *header;
    struct vbios_desc  *desc;
    u32  now_offset;
    u32 i = 0;
    u8 save_crc = 0;
    header =(struct vbios_header *)this->get_data_by_type(desc_header);
    if (!header) {
       return 1;
    }
    now_offset = header->data_offset;

    for(bliterator=vbt_block_list.begin();bliterator!=vbt_block_list.end();bliterator++){
       desc=bliterator->desc;
       if (save_crc == 0) {
           if ( (now_offset+desc->size + 4 ) >= 0x1e000) {
               now_offset += desc->size+4;
               save_crc  = 1;
           }
       }
       desc->offset = now_offset;
       now_offset += desc->size;
       i++;
    }
    return 0;
}

int Widget::fill_vbios_data(u8 *mem_in)
{
   struct block_list *node;
   struct vbios_header *header;
   struct vbios_desc   *desc, *desc_p, desc_eof = {.type = desc_max,};
   struct loongson_vbios *vbios_header = (struct loongson_vbios *)mem_in;

   if(!mem_in)
       qDebug("mem_in nullptr!\n");
    u8 *data_p;
    u32 size_desc = sizeof(*desc);
    header = (struct vbios_header *) this->get_data_by_type(desc_header);
    if (!header) {
        qDebug("fill data header error!\n");
            exit(1);
    }

    desc_p =(struct vbios_desc *)(mem_in+header->desc_offset);
    for(bliterator=vbt_block_list.begin();bliterator!=vbt_block_list.end();bliterator++)
    {
        desc=bliterator->desc;//desc = node->desc;
        data_p = mem_in+ desc->offset;
        memcpy(desc_p, desc, size_desc);
        memcpy(data_p, bliterator->data, desc->size);
        desc_p++;
    }

    memcpy(desc_p, &desc_eof, size_desc);
    sprintf(vbios_header->title,"Loongson-VBIOS");
    vbios_header->crtc_num = num;
    vbios_header->version_major = 1;
    vbios_header->version_minor = 0;
    return 0;
}

int Widget::vbios_assembly_vbios()
{
    u32 *crc;
    int ret;
    ret = this->fill_vbios_header();
    this->fill_vbios_desc();
    this->fill_vbios_data((u8 *)ldevit->vbios);
    crc = (unsigned int *)((unsigned char *)ldevit->vbios + 0x1E000 - 0x4);
    *crc = crc32(0,(char *)ldevit->vbios,0x1E000 - 0x4);
    return 0;
}

void Widget::saveVbios(const QString &filename)
{
    int index = 1;
    QFile vbios(filename);
    vbios.open(QFile::WriteOnly);

    this->loongson_vbios_simu_data(index);
    vbios.write((char*)ldevit->vbios,256*1024);
    vbios.flush();
    vbios.close();
    return;
}

void Widget::on_encoder_config_btn_clicked()
{
    config_page1->config_type1 = all[0].encoder->config_type;
    if(config_page1->config_type1 == encoder_os_config){
        config_page1->ui->tableWidget->setColumnHidden(0,false);
        config_page1->ui->tableWidget->setColumnHidden(1,false);
    }
    config_page1->show();
}

void Widget::on_connector_edid_clicked()
{
    edid_page1->show();
}


void Widget::on_save_btn_clicked()
{
    QString file_name;
    file_name = QFileDialog::getSaveFileName(this,"保存文件",".","vbios(*.bin);;binary(*.bin);;All files(*.*);;",nullptr,
                                                      QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
    if(file_name.isEmpty())
         return;
    saveVbios(file_name);
    return;
}

void INIT_LIST_HEAD(struct list_head *list)
{
    list->next = list;
    list->prev = list;
    return;
}

void plist_add(struct list_head *newn,struct list_head *phead)
{
    __plist_add(newn,phead,phead->next);
    return;
}

void __plist_add(struct list_head *newn, struct list_head * prev,struct list_head *next)
{
    next->prev = newn;
    newn->next = next;
    newn->prev = prev;
    prev->next = newn;
    return;
}

void __list_del(struct list_head *prev,list_head *next)
{
   next->prev = prev;
   prev->next = next;
   return;
}

void list_del(struct list_head *entry)
{
    __list_del(entry->prev,entry->next);
    entry->next = nullptr;
    entry->prev = nullptr;
    return;
}

int print_func(struct loongson_device *ldev)
{
    struct desc_node *node,*tmp;
    list_for_each_entry_safe(node,tmp,&ldev->desc_list,head){
        if(parse_flag){
            fprintf(fp,"===========================%s ====================\n",desc_name[node->desc->type]);
        }

        node->parse(node,nullptr);
    }
    return 0;
}

int Widget::loongson_vbios_init(struct loongson_device *ldev)
{
    u32 ret = 0;
    int crtc_num = 0;
    struct loongson_vbios *vbios_header = (struct loongson_vbios *)ldev->vbios;
    crtc_num = vbios_header->crtc_num;
    num_link  = crtc_num;
    this->loongson_vbios_crc_check(ldev->vbios);
    this->loongson_vbios_title_check((struct loongson_vbios *)ldev->vbios);

    INIT_LIST_HEAD(&ldev->desc_list);
    ret = this->parse_vbios_desc(ldev);
    print_func(ldev);
    return 0;
}

int Widget::loongson_vbios_exit(struct loongson_device *ldev)
{
    free_desc_list(ldev);
    free(ldev->vbios);
    return 0;
}

int Widget::loongson_vbios_crc_check(void *vbios)
{
    unsigned int crc;

    crc = this->lscrc32(0, (unsigned char*)vbios,  0x1e000 - 0x4);
    if (*(unsigned int *)((unsigned char*)vbios + 0x1e000 - 0x4) != crc) {
        return 1;
    }
    return 0;
}

void Widget::kernel_make_table(void)
{
    int i, j;
    have_table = 1;
    for (i = 0; i < 256; i++)
        for (j = 0, table[i] = i; j < 8; j++)
            table[i] = (table[i]>>1)^((table[i]&1)? POLYNOMIAL : 0);
    return;
}

uint Widget::lscrc32(uint crc, unsigned char *buff, int len)
{
    int i;
    if (!have_table) this->kernel_make_table();
    crc = ~crc;
    for (i = 0; i < len; i++)
        crc = (crc >> 8) ^ table[(crc ^ buff[i]) & 0xff];
    return ~crc;
}

bool parse_vbios_pwm(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    bool ret = true;
    static int index_pwm = 0;
    struct vbios_pwm *pwm = (struct vbios_pwm *)pthis->data;
    if(index_pwm == num_link)
        index_pwm = 0;

    if(!parse_flag){
        parse_all[index_pwm].pwm->feature = pwm->feature;
        parse_all[index_pwm].pwm->peroid = pwm->peroid;
        parse_all[index_pwm].pwm->polarity = pwm->polarity;
        parse_all[index_pwm].pwm->pwm = pwm->pwm;
        parse_all[index_pwm].pwm_index = pthis->desc->index;
    }

    if(parse_flag)
        fprintf(fp,"Vbios pwm_index-%d feature-%#x pwm-%d polarity-%d frequency-%d\n",
            pthis->desc->index, pwm->feature, pwm->pwm, pwm->polarity, pwm->peroid);

    ++index_pwm;

    return ret;
}

bool parse_vbios_backlight(struct desc_node *pthis, struct vbios_cmd *cmd)
{
    static int index_bl = 0;
    if(num_link == index_bl)//reset 0
        index_bl = 0;

    char *vbios_backlight_type[]= {
        "bl_unuse",
        "bl_ec",
        "bl_pwm"
    };

    struct vbios_backlight *bl =(struct vbios_backlight *)pthis->data;
    if(!parse_flag){
        parse_all[index_bl].bl->type = bl->type;
        parse_all[index_bl].bl->used = bl->used;
        parse_all[index_bl].bl_index = pthis->desc->index;
    }

    if(parse_flag)
        fprintf(fp,"Vbios bl_index-%d bl_uesd-%d type-%s \n", pthis->desc->index,bl->used,vbios_backlight_type[bl->type]);

    ++index_bl;
    return true;
}

bool parse_vbios_header(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    return true;
}

bool parse_vbios_crtc(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    bool ret = true;
    struct vbios_crtc *crtc = (struct vbios_crtc *)pthis->data;
    static int index_crtc = 0;

    if(num_link == index_crtc)//reset 0
        index_crtc = 0;


    if(!parse_flag){
        parse_all[index_crtc].crtc->feature = crtc->feature;
        parse_all[index_crtc].crtc->crtc_id = crtc->crtc_id;
        parse_all[index_crtc].crtc->encoder_id = crtc->encoder_id;
        parse_all[index_crtc].crtc->max_freq = crtc->max_freq;
        parse_all[index_crtc].crtc->max_height = crtc->max_height;
        parse_all[index_crtc].crtc->max_width = crtc->max_width;

        }

    if(parse_flag)
        fprintf(fp,"Vbios crtc-%d feat %#x dvo %d enc %d \n max_fre %d max_wid %d max_height %d  is_vb_timing %d \n",
            pthis->desc->index, crtc->feature, crtc->crtc_id, crtc->encoder_id, crtc->max_freq,
            crtc->max_width, crtc->max_height, crtc->is_vb_timing);

    ++index_crtc;
    return ret;
}


bool parse_vbios_i2c(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    int i;
    char * i2c_type[] = {"i2c_cpu","i2c_gpio","i2c_unkown"};
    struct vbios_i2c *i2c = (struct vbios_i2c *)pthis->data;
    u32 inum = pthis->desc->size/(sizeof(*i2c));
    static int index_i2c = 0;

    if(num_link == index_i2c)
        index_i2c = 0;

    for (i = 0; (i < inum && i < 16); i++) {
        if(parse_flag)
            fprintf(fp,"| Vbios I2c-%d feature-%#x id-%d type-%s |\n",
                pthis->desc->index, i2c->feature, i2c->id, i2c_type[i2c->type]);
        if(!parse_flag){
            parse_all[index_i2c].i2c->feature = i2c->feature;
            parse_all[index_i2c].i2c->id = i2c->id;
            parse_all[index_i2c].i2c->type = i2c->type;
        }
        ++index_i2c;
        i2c++;
    }

    return true;
}


bool parse_vbios_connector(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    struct vbios_connector *con = (struct vbios_connector *)pthis->data;
    int i = 0;
    static int index_connector = 0;
    int index = -1;

    if(num_link == index_connector) //reset 0
        index_connector = 0;

    char *conn_type[] = {
        "connector_unknown",
        "connector_vga",
        "connector_dvii",
        "connector_dvid",
        "connector_dvia",
        "connector_composite",
        "connector_svideo",
        "connector_lvds",
        "connector_component",
        "connector_9pindin",
        "connector_displayport",
        "connector_hdmia",
        "connector_hdmib",
        "connector_tv",
        "connector_edp",
        "connector_virtual",
        "connector_dsi",
        "connector_dpi"
    };

    char *hotplug[] = {
        "disable",
        "polling",
        "irq",
        "hotplug_max",
    };
    char *edid_method[] = {
        "via_nullptr",
        "via_i2c",
        "via_vbios",
        "via_encoder",
        "via_max",
    };

    char* irq_placement[] ={
        "_3A4000",
        "_7A1000",
    };

    switch (con->edid_method) {
        case via_i2c:
            index = 1;
        break;
        case via_vbios:
            index = 2;
        break;
        default:
        break;
    }

    if(!parse_flag){
        parse_all[index_connector].connector->feature = con->feature;
        parse_all[index_connector].connector->i2c_id = con->i2c_id;
        parse_all[index_connector].connector->type = con->type;
        parse_all[index_connector].connector->hotplug = con->hotplug;
        parse_all[index_connector].connector->edid_method = con->edid_method;
    }

    if(parse_flag)
        fprintf(fp,"Vbios conn-%d feature %#x i2c-%d type %s \n  hpd %s edid %s \n",
                pthis->desc->index, con->feature, con->i2c_id, conn_type[con->type] ,
                hotplug[con->hotplug],edid_method[index]);

  if(edid_method[con->edid_method] == "via_vbios")
    {
        for(i=0;i<sizeof(con->internal_edid);i++){
            if(parse_flag)
                fprintf(fp,"internal_edid[%d]=%x ",i,con->internal_edid[i]);
            if(i%7==0){
                if(parse_flag)
                    fprintf(fp,"\n");
            }
        }
        if(parse_flag)
             fprintf(fp,"\n");
    }

  /*
    if(hotplug[con->hotplug] == "irq")
        {
        if(parse_flag)
            fprintf(fp,"gpio_placement %s irq_gpio %d\n",
                irq_placement[con->irq_placement],con->irq_gpio);
        if(!parse_flag){
            parse_all[index_connector].connector->irq_gpio = con->irq_gpio;
            parse_all[index_connector].connector->irq_placement = con->irq_placement;
        }
    }else {
         if(parse_flag)
            fprintf(fp,"irq_gpio %d\n",con->irq_gpio);
    }
*/
    ++index_connector;

    return 0;
}

bool parse_vbios_encoder(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    bool ret = true;
    struct vbios_encoder *enc = (struct vbios_encoder *)pthis->data;
    static int index_encoder = 0;
    int index = -1;
    int type_index = -1;

    if(num_link == index_encoder)
        index_encoder = 0;

    char *enc_type[]={
        "encoder_none",
        "encoder_dac",
        "encoder_tmds",
        "encoder_lvds",
        "encoder_tvdac",
        "encoder_virtual",
        "encoder_dsi",
        "encoder_dpmst",
        "encoder_dpi"
    };

    char *encoder_config[] = {
        "encoder_vbios_config",
        "encoder_legacy",
        "encoder_kernel", //bios config encoder
        "encoder_type_max",
    };

    char *chip[] = {
        "Unknown",
        "Internal_Dvo",
        "Internal_Hdmi",
        "Vga_Ch7055",
        "Vga_Adv7125",
        "Dvi_Tfp410",
        "Hdmi_It66121",
        "Hdmi_Sil9022",
        "Hdmi_Lt8618",
        "Hdmi_Ms7210",
        "Edp_Ncs8805",
    };

    switch(enc->chip)
    {
       case 0x00:
           index = 0;
       break;
       case 0x01:
            index = 1;
       break;
       case 0x02:
           index = 2;
       break;
       case 0x10:
           index = 3;
       break;
       case 0x11:
           index = 4;
       break;
       case 0x20:
           index = 5;
       break;
       case 0x30:
           index = 6;
       break;
       case 0x31:
           index = 7;
       break;
       case 0x32:
           index = 8;
       break;
       case 0x33:
           index = 9;
       break;
       case 0x40:
           index = 10;
       break;
       default:
       break;
    }

    switch (enc->config_type) {
    case encoder_os_config:
            type_index = 0;
        break;
    case encoder_timing_filling:
            type_index = 1;
        break;
    case encoder_kernel_driver:
            type_index = 2;
        break;
    default:
        break;
    }

    if(!parse_flag){
        parse_all[index_encoder].encoder->feature = enc->feature;
        parse_all[index_encoder].encoder->i2c_id = enc->i2c_id;
        parse_all[index_encoder].encoder->connector_id = enc->connector_id;
        parse_all[index_encoder].encoder->type = enc->type;
        parse_all[index_encoder].encoder->config_type = enc->config_type;
        parse_all[index_encoder].encoder->chip_addr = enc->chip_addr;
        parse_all[index_encoder].encoder->chip =  (enum encoder_object)index;
      }   

    if(parse_flag)
        fprintf(fp,"Vbios encod-%d feature %#x i2c %d conn %d type %s conf %s \n",
            pthis->desc->index , enc->feature, enc->i2c_id, enc->connector_id,
            enc_type[enc->type],encoder_config[type_index]);
    if(parse_flag)
        fprintf(fp,"encoder===> chip_addr %#x chip %s\n",enc->chip_addr,chip[index]);

    ++index_encoder;
    return ret;
}

bool parse_vbios_cfg_encoder(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    bool ret = true;
    static int index_cfg = 0;
    struct vbios_cfg_encoder *vbios_cfg_encoder;
    struct vbios_desc *vbt_desc;
    struct vbios_conf_reg *config_reg;
    u32 cnum, reg_num = 0, size, i = 0 , j=0;

    if(num_link == index_cfg)
        index_cfg = 0;

    vbios_cfg_encoder = (struct vbios_cfg_encoder *)pthis->data;
    vbt_desc = (struct vbios_desc *)pthis->desc;

    size = sizeof(struct vbios_cfg_encoder);
    cnum = pthis->desc->size/size;

    if(parse_flag)
        fprintf(fp,"Vbios vbios_config_encode-%d \n",pthis->desc->index);

    for (i = 0; i < cnum; i++) {
        if(parse_flag)
            fprintf(fp,"reg_nums-%d hdisplay-%d	vdisplay-%d \n",
                    vbios_cfg_encoder->reg_num, vbios_cfg_encoder->hdisplay,vbios_cfg_encoder->vdisplay);
        config_reg = vbios_cfg_encoder->config_regs;
        reg_num  = vbios_cfg_encoder->reg_num;

        if(!parse_flag){
            parse_all[index_cfg].cfg[i]->hdisplay = vbios_cfg_encoder->hdisplay;
            parse_all[index_cfg].cfg[i]->vdisplay = vbios_cfg_encoder->vdisplay;
        }

        for (j=0; j < reg_num; j++) {
            if(parse_flag)
                fprintf(fp,"| addr-%02x reg-%#03x val-%#03x |\n",config_reg->dev_addr, config_reg->reg, config_reg->value);
            config_reg++;
        }
        vbios_cfg_encoder++;
    }

    ++index_cfg;

    return ret;
}

void Widget::free_desc_list(loongson_device *ldev)
{
    struct desc_node *node,*tmp;

    list_for_each_entry_safe(node,tmp,&ldev->desc_list,head){
        list_del(&node->head);
        free(node);
    }
    return;
}

u32 Widget::parse_vbios_desc(struct loongson_device *ldev)
{
    u32 ret = 0;
    struct vbios_desc *desc;
    u16 type = 0;
    u8 *vbios = (u8 *)ldev->vbios;
    struct loongson_vbios *header = (struct loongson_vbios *)ldev->vbios;
    desc = (struct vbios_desc *)(vbios + VBIOS_DESC_OFFSET);

    while (1) {
        type = desc->type;
        if (type == desc_max)
            break;

        ret = insert_desc_list(ldev,desc);
        if (ret)
            qDebug("Parse T-%d V-%d failed[%d]\n",
                      desc->ver, desc->type, ret);
        desc++;
    }
    return ret;
}

u32 insert_desc_list(struct loongson_device *ldev ,struct vbios_desc *vb_desc)
{
    struct desc_node *node;
    parse_func func = nullptr;
    int type = -1;
    node = (struct desc_node *)malloc(sizeof(*node));
    if (!node) {
        printf("%s no mme",__func__);
        exit(-1);
    }

    func = get_parse_func(vb_desc);
    node->parse = func;
    node->desc  = (struct vbios_desc *)vb_desc;
    node->data  = ((u8 *)ldev->vbios + vb_desc->offset);
    plist_add(&node->head,&ldev->desc_list);
    return 0;
}

parse_func get_parse_func(struct vbios_desc *vb_desc)
{
    int i;
    u32 type = vb_desc->type;
    u32 ver  = vb_desc->ver;
    parse_func   func = parse_vbios_default;
    u32 tt_num = ARRAY_SIZE(tables);

    for (i = 0; i < tt_num; i++) {
        if ((tables[i].ver == ver) && (tables[i].type == type)) {
            func = tables[i].func;
            break;
        }
    }
    return func;
}

bool parse_vbios_default(struct desc_node *pthis,struct vbios_cmd *cmd)
{
    struct vbios_desc *vb_desc;
    vb_desc = pthis->desc;
    if(parse_flag)
        fprintf(fp,"Current descriptor[T-%#x][V-%d] cannot be interprete.\n",
            vb_desc->type, vb_desc->ver);

   return false;
}

int Widget::loongson_vbios_title_check(struct loongson_vbios *vbios)
{
    char * title="Loongson-VBIOS";
    int i;
    i = 0;
    while(*title != '\0' && i <= 15){
        if(vbios->title[i++] != *title){
            qDebug("VBIOS title is wrong,use default setting!\n");
            return 1;
        }
        title++;
    }
    return 0;
}

int Widget::loongson_get_file(u32 size,struct loongson_device *ldev,QString fname)
{
    FILE *fd;
    char *filename = nullptr;
    QFileInfo fi;

    fi = QFileInfo(fname);
    QString file_path = fi.absoluteFilePath();
    QByteArray ba = file_path.toLatin1();
    filename = ba.data();

    if((fd=fopen(filename,"rb")) == nullptr){
        qDebug("The file %s can not be opened.\n",filename);
        exit(0);
    }

    fseek(fd, 0 ,SEEK_SET);
    fread(ldev->vbios,size,1,fd);

    fflush(fd);
    fclose(fd);
    return 0;
}


void Widget::on_check_btn_clicked()
{
    QString name;
    struct loongson_device *ldev;
    name = QFileDialog::getOpenFileName(this,"打开",
                            QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation),"*(*.bin)");
    if(name.isEmpty())
         return;

    QString filename=QFileDialog::getSaveFileName(this,tr("保存解析文件"),tr("*.txt"),nullptr);
    if(filename.isEmpty())
        return;

    const char *fname;
    QByteArray ba = filename.toLatin1();
    fname = ba.data();

    fp = fopen(fname,"w");  //char*
    ldev = (struct loongson_device *)malloc(sizeof(struct loongson_device));
    ldev->vbios = (u8*)malloc(VBIOS_SIZE);
    parse_flag = 1;

    if(!ldev->vbios){
        printf("alloc vbios falling!\n");
    }
    memset(ldev->vbios,(u8)-1,VBIOS_SIZE);
    this->loongson_get_file(VBIOS_SIZE,ldev,name);
    this->loongson_vbios_init(ldev);
    this->loongson_vbios_exit(ldev);
}

void Widget::clear_data_link1()
{
    ui->crtc_id_spin->setValue(0);
    ui->crtc_freq_spin->setValue(0);

    ui->encoder_type_combo->setCurrentIndex(0);
    ui->encoder_config_btn->hide();

    hexspin1->setValue(0);
    ui->chip_combo->setCurrentIndex(0);
    ui->config_type_combo->setCurrentIndex(0);

    ui->connect_type_combo->setCurrentIndex(0);
    ui->edid_method_combo->setCurrentIndex(0);
    ui->connector_edid->hide();

    ui->hotplug_combo->setCurrentIndex(0);
    ui->irq_placement_combo->setCurrentIndex(0);
    ui->irq_placement_combo->hide();
    ui->irq_gpio_spin->setValue(0);
    ui->irq_gpio_spin->hide();

    ui->pwm_id_spin->setValue(0);
    ui->polarity_spin->setValue(0);
    ui->peroid_spin->setValue(0);
}

void Widget::load_crtc_to_ui()
{
    //only enable DVO0
    if(dvo_flag == 0){
        ui->crtc_id_spin->setValue(parse_all[0].crtc->crtc_id);
        ui->crtc_freq_spin->setValue(parse_all[0].crtc->max_freq);

        ui->crtc_id_spin_2->setValue(0);
        ui->freq_spin_2->setValue(0);
    }

    //only enable DVO1
    if(dvo_flag == 1){
        ui->crtc_id_spin_2->setValue(parse_all[0].crtc->crtc_id);
        ui->freq_spin_2->setValue(parse_all[0].crtc->max_freq);

        ui->crtc_id_spin->setValue(0);
        ui->crtc_freq_spin->setValue(0);
    }

    //enable DVO0 and DVO1  //modify
    if(dvo_flag == 2){
        ui->crtc_id_spin->setValue(parse_all[1].crtc->crtc_id);
        ui->crtc_freq_spin->setValue(parse_all[1].crtc->max_freq);

        ui->crtc_id_spin_2->setValue(parse_all[0].crtc->crtc_id);
        ui->freq_spin_2->setValue(parse_all[0].crtc->max_freq);
    }
    return;
}

void Widget::load_encoder_to_ui()
{
    int type1 = 0,type2 = 0;
    int config_type = 0,config_type2 = 0;
    type1 = parse_all[0].encoder->type;
    type2 = parse_all[1].encoder->type;
    config_type = parse_all[0].encoder->config_type;
    config_type2 = parse_all[1].encoder->config_type;

    if(parse_all[0].encoder->type && parse_all[0].crtc->crtc_id == 0){
       dvo_flag = 0;
       ui->dvo0_btn->setChecked(true);
       ui->tabWidget->setTabEnabled(0,true);
       ui->tabWidget->setTabEnabled(1,false);

       ui->encoder_type_combo->setCurrentIndex(type1);
       if(config_type == 1){
           ui->encoder_config_btn->show();
           ui->config_type_combo->setCurrentIndex(0);
       }else if(config_type == 3){
           ui->encoder_config_btn->hide();
           ui->config_type_combo->setCurrentIndex(1);
       }else if(config_type == 4){
           ui->encoder_config_btn->hide();
           ui->config_type_combo->setCurrentIndex(2);
       }

       hexspin1->setValue(parse_all[0].encoder->chip_addr);
       ui->chip_combo->setCurrentIndex(parse_all[0].encoder->chip);

       ui->encoder_type_combo_2->setCurrentIndex(0);
       ui->config_type_combo_2->setCurrentIndex(1);
       hexspin2->setValue(0);
       ui->chip_combo_2->setCurrentIndex(0);
       ui->encoder_config_btn_2->hide();

    }else if(parse_all[0].encoder->type && parse_all[0].crtc->crtc_id == 1 && parse_all[1].encoder->type == 0){
       dvo_flag = 1;
       ui->dvo1_btn->setChecked(true);
       ui->tabWidget->setTabEnabled(0,false);
       ui->tabWidget->setTabEnabled(1,true);

       ui->encoder_type_combo_2->setCurrentIndex(type1);
       if(config_type == 1){
           ui->encoder_config_btn_2->show();
           ui->config_type_combo_2->setCurrentIndex(0);
       }else if(config_type == 3){
           ui->encoder_config_btn_2->hide();
           ui->config_type_combo_2->setCurrentIndex(1);
       }else if(config_type == 4){
           ui->encoder_config_btn_2->hide();
           ui->config_type_combo_2->setCurrentIndex(2);
       }

       hexspin2->setValue(parse_all[0].encoder->chip_addr);
       ui->chip_combo_2->setCurrentIndex(parse_all[0].encoder->chip);

       ui->encoder_type_combo->setCurrentIndex(0);
       ui->config_type_combo->setCurrentIndex(1);
       hexspin1->setValue(0);
       ui->chip_combo->setCurrentIndex(0);
       ui->encoder_config_btn->hide();

    }else if(parse_all[0].encoder->type && parse_all[1].encoder->type){
        dvo_flag = 2;
        ui->dvo_all_btn->setChecked(true);
        ui->tabWidget->setTabEnabled(0,true);
        ui->tabWidget->setTabEnabled(1,true);
        ui->encoder_type_combo->setCurrentIndex(parse_all[1].encoder->type);
        ui->encoder_type_combo_2->setCurrentIndex(parse_all[0].encoder->type);

        if(config_type2 == 1){
               ui->encoder_config_btn->show();
               ui->config_type_combo->setCurrentIndex(0);
           }else if(config_type2 == 3){
               ui->encoder_config_btn->hide();
               ui->config_type_combo->setCurrentIndex(1);
           }else if(config_type2 == 4){
               ui->encoder_config_btn->hide();
               ui->config_type_combo->setCurrentIndex(2);
         }

        hexspin1->setValue(parse_all[1].encoder->chip_addr);
        ui->chip_combo->setCurrentIndex(parse_all[1].encoder->chip);

        hexspin2->setValue(parse_all[0].encoder->chip_addr);
        ui->chip_combo_2->setCurrentIndex(parse_all[0].encoder->chip);

        if(config_type == 1){
               ui->encoder_config_btn_2->show();
               ui->config_type_combo_2->setCurrentIndex(0);
           }else if(config_type == 3){
               ui->encoder_config_btn_2->hide();
               ui->config_type_combo_2->setCurrentIndex(1);
           }else if(config_type == 4){
               ui->encoder_config_btn_2->hide();
               ui->config_type_combo_2->setCurrentIndex(2);
         }
    }

    return;
}

void Widget::load_connector_to_ui()
{
    if(dvo_flag == 0){
        ui->connect_type_combo->setCurrentIndex(parse_all[0].connector->type);
        ui->hotplug_combo->setCurrentIndex(parse_all[0].connector->hotplug);
        if(parse_all[0].connector->hotplug == 2){
            ui->irq_placement_la->show();
            ui->irq_placement_combo->show();

            ui->irq_gpio_lb->show();
            ui->irq_gpio_spin->show();
        }

        if(parse_all[0].connector->edid_method == via_i2c)
            ui->edid_method_combo->setCurrentIndex(0);
        else if(parse_all[0].connector->edid_method == via_vbios){
            ui->edid_method_combo->setCurrentIndex(1);
            ui->connector_edid->show();
        }

        ui->irq_placement_combo->setCurrentIndex(parse_all[0].connector->irq_placement);
        if(parse_all[0].connector->irq_gpio == -1)
            ui->irq_gpio_spin->setValue(0);
        else
            ui->irq_gpio_spin->setValue(parse_all[0].connector->irq_gpio);

        //clear dvo1
         ui->connect_type_combo_2->setCurrentIndex(0);
         ui->hotplug_combo_2->setCurrentIndex(0);
         ui->edid_method_combo_2->setCurrentIndex(0);
         ui->irq_placement_combo_2->setCurrentIndex(0);
         ui->irq_gpio_spin_2->setValue(0);

         ui->connector_edid_2->hide();
         ui->irq_plament_lab2->hide();
         ui->irq_placement_combo_2->hide();
         ui->irq_gpio_lab2->hide();
         ui->irq_gpio_spin_2->hide();

    }else if(dvo_flag == 1){
        ui->connect_type_combo_2->setCurrentIndex(parse_all[0].connector->type);
        ui->hotplug_combo_2->setCurrentIndex(parse_all[0].connector->hotplug);
        if(parse_all[0].connector->hotplug == 2){
            ui->irq_plament_lab2->show();
            ui->irq_placement_combo_2->show();

            ui->irq_gpio_lab2->show();
            ui->irq_gpio_spin_2->show();
        }

        if(parse_all[0].connector->edid_method == via_i2c)
            ui->edid_method_combo_2->setCurrentIndex(0);
        else if(parse_all[0].connector->edid_method == via_vbios){
            ui->edid_method_combo_2->setCurrentIndex(1);
            ui->connector_edid_2->show();
        }

        ui->irq_placement_combo_2->setCurrentIndex(parse_all[0].connector->irq_placement);
        if(parse_all[0].connector->irq_gpio == -1)
            ui->irq_gpio_spin_2->setValue(0);
        else
            ui->irq_gpio_spin_2->setValue(parse_all[0].connector->irq_gpio);

        ui->connect_type_combo->setCurrentIndex(0);
        ui->hotplug_combo->setCurrentIndex(0);
        ui->edid_method_combo->setCurrentIndex(0);
        ui->irq_placement_combo->setCurrentIndex(0);
        ui->irq_gpio_spin->setValue(0);

        ui->connector_edid->hide();
        ui->irq_placement_la->hide();
        ui->irq_placement_combo->hide();
        ui->irq_gpio_lb->hide();
        ui->irq_gpio_spin->hide();


    }else if(dvo_flag == 2){
        ui->connect_type_combo->setCurrentIndex(parse_all[1].connector->type);
        ui->hotplug_combo->setCurrentIndex(parse_all[1].connector->hotplug);
        if(parse_all[1].connector->edid_method == via_i2c)
            ui->edid_method_combo->setCurrentIndex(0);
        else if(parse_all[1].connector->edid_method == via_vbios)
            ui->edid_method_combo->setCurrentIndex(1);
        ui->irq_placement_combo->setCurrentIndex(parse_all[1].connector->irq_placement);
        if(parse_all[1].connector->irq_gpio == -1)
            ui->irq_gpio_spin->setValue(0);
        else
            ui->irq_gpio_spin->setValue(parse_all[1].connector->irq_gpio);

        ui->connect_type_combo_2->setCurrentIndex(parse_all[0].connector->type);
        ui->hotplug_combo_2->setCurrentIndex(parse_all[0].connector->hotplug);
        if(parse_all[0].connector->edid_method == via_i2c)
            ui->edid_method_combo_2->setCurrentIndex(0);
        else if(parse_all[0].connector->edid_method == via_vbios)
            ui->edid_method_combo_2->setCurrentIndex(1);
        ui->irq_placement_combo_2->setCurrentIndex(parse_all[0].connector->irq_placement);
        if(parse_all[0].connector->irq_gpio == -1)
            ui->irq_gpio_spin_2->setValue(0);
        else
            ui->irq_gpio_spin_2->setValue(parse_all[0].connector->irq_gpio);
    }
    return;
}

void Widget::load_i2c_to_ui()
{
    if(dvo_flag == 0){
        ui->i2c_type_combo->setCurrentIndex(parse_all[0].i2c->type);
        ui->i2c_type_combo_2->setCurrentIndex(0);
    }else if(dvo_flag == 1){
        ui->i2c_type_combo_2->setCurrentIndex(parse_all[0].i2c->type);
        ui->i2c_type_combo->setCurrentIndex(0);
    }else if(dvo_flag == 2){
        ui->i2c_type_combo->setCurrentIndex(parse_all[1].i2c->type);
        ui->i2c_type_combo_2->setCurrentIndex(parse_all[0].i2c->type);
    }
    return;
}

void Widget::load_pwm_to_ui()
{
    if(dvo_flag == 0){
          ui->pwm_id_spin->setValue(parse_all[0].pwm->pwm);
          ui->polarity_spin->setValue(parse_all[0].pwm->polarity);
          ui->peroid_spin->setValue(parse_all[0].pwm->peroid);

          //clear dvo1
          ui->pwm_id_spin_2->setValue(0);
          ui->polarity_spin_2->setValue(0);
          ui->peroid_spin_2->setValue(0);

    }else if(dvo_flag == 1){
          ui->pwm_id_spin_2->setValue(parse_all[0].pwm->pwm);
          ui->polarity_spin_2->setValue(parse_all[0].pwm->polarity);
          ui->peroid_spin_2->setValue(parse_all[0].pwm->peroid);

          //clear
          ui->pwm_id_spin->setValue(0);
          ui->polarity_spin->setValue(0);
          ui->peroid_spin->setValue(0);

    }else if(dvo_flag == 2){
        if(parse_all[0].pwm->pwm && parse_all[1].pwm->pwm){
            ui->pwm_id_spin->setValue(parse_all[1].pwm->pwm);
            ui->pwm_id_spin_2->setValue(parse_all[0].pwm->pwm);

            ui->polarity_spin->setValue(parse_all[1].pwm->polarity);
            ui->polarity_spin_2->setValue(parse_all[0].pwm->polarity);

            ui->peroid_spin->setValue(parse_all[1].pwm->peroid);
            ui->peroid_spin_2->setValue(parse_all[0].pwm->peroid);
        }else if(parse_all[0].pwm->pwm && parse_all[0].pwm_index == 0){
            ui->pwm_id_spin->setValue(parse_all[0].pwm->pwm);
            ui->polarity_spin->setValue(parse_all[0].pwm->polarity);
            ui->peroid_spin->setValue(parse_all[0].pwm->peroid);
        }else if(parse_all[0].pwm->pwm && parse_all[0].pwm_index == 1){
            ui->pwm_id_spin_2->setValue(parse_all[0].pwm->pwm);
            ui->polarity_spin_2->setValue(parse_all[0].pwm->polarity);
            ui->peroid_spin_2->setValue(parse_all[0].pwm->peroid);
        }
    }
    return;
}

void Widget::load_backlight_to_ui()
{
    if(dvo_flag == 0){
          ui->bl_type_combo->setCurrentIndex(parse_all[0].bl->type);
          ui->bl_used_spin->setValue(parse_all[0].bl->used);
          //clear dvo1
          ui->bl_type_combo2->setCurrentIndex(0);
          ui->bl_used_spin2->setValue(0);

    }else if(dvo_flag == 1){
          ui->bl_type_combo2->setCurrentIndex(parse_all[0].bl->type);
          ui->bl_used_spin2->setValue(parse_all[0].bl->used);

          //clear
          ui->bl_type_combo->setCurrentIndex(0);
          ui->bl_used_spin->setValue(0);

    }else if(dvo_flag == 2){
        if(parse_all[1].bl->type && parse_all[0].bl->type){
            ui->bl_type_combo->setCurrentIndex(parse_all[1].bl->type);
            ui->bl_used_spin->setValue(parse_all[1].bl->used);

            ui->bl_type_combo2->setCurrentIndex(parse_all[0].bl->type);
            ui->bl_used_spin2->setValue(parse_all[0].bl->used);

        }else if(parse_all[0].bl->type && parse_all[0].bl_index == 0){
            ui->bl_type_combo->setCurrentIndex(parse_all[0].bl->type);
            ui->bl_used_spin->setValue(parse_all[0].bl->used);
        }else if(parse_all[0].bl->type && parse_all[0].bl_index == 1){
            ui->bl_type_combo2->setCurrentIndex(parse_all[0].bl->type);
            ui->bl_used_spin2->setValue(parse_all[0].bl->used);
        }
    }
    return;
}

void Widget::load_cfg_to_ui()
{
    int cfg_grounp1 = 0;
    int cfg_grounp2 = 0;

    for(int i=0;i<10;i++){ //3
        if(parse_all[0].cfg[i]->hdisplay)
            cfg_grounp1++;
        else
            break;
    }

    for(int j=0;j<10;j++){ //2
        if(parse_all[1].cfg[j]->hdisplay)
            cfg_grounp2++;
        else
            break;
    }

    for(int i=0;i<10;i++){
        if(config_page1->ui->tableWidget->item(i,0)){
            config_page1->ui->tableWidget->takeItem(i,0);
            config_page1->ui->tableWidget->takeItem(i,1);
        }else
            break;
     }

     for(int i=0;i<10;i++){
        if(config_page2->ui->tableWidget->item(i,0)){
            config_page2->ui->tableWidget->takeItem(i,0);
            config_page2->ui->tableWidget->takeItem(i,1);;
        }else
            break;
     }

   if(cfg_grounp1){
        for(int t=0;t<cfg_grounp1;t++){
          QTableWidgetItem *item1ht = new QTableWidgetItem();
          QTableWidgetItem *item1vt= new QTableWidgetItem();

          itemh.append(item1ht);
          itemv.append(item1vt);
        }
  }

    if(cfg_grounp2){
        for(int k=0;k<cfg_grounp2;k++){
            QTableWidgetItem *item2hk = new QTableWidgetItem();
            QTableWidgetItem *item2vk = new QTableWidgetItem();
            itemh2.append(item2hk);
            itemv2.append(item2vk);
        }
    }

    if(!dvo_flag){ //dvo_flag
        for(int i=0;i<cfg_grounp1;i++){
            QString hstr = QString::number(parse_all[0].cfg[i]->hdisplay);
            QString vstr = QString::number(parse_all[0].cfg[i]->vdisplay);

            itemh[i]->setText(hstr);
            config_page1->ui->tableWidget->setItem(i,0,itemh[i]);
            itemv[i]->setText(vstr);
            config_page1->ui->tableWidget->setItem(i,1,itemv[i]);

            config_page1->Bar1[i]->setValue(100);
            QString name = hstr+"_"+vstr;
            config_page1->LineEd1[i]->setText(name);
        }
    }else if(dvo_flag == 1){
        for(int i=0;i<cfg_grounp1;i++){
            QString hstr2 = QString::number(parse_all[0].cfg[i]->hdisplay);
            QString vstr2 = QString::number(parse_all[0].cfg[i]->vdisplay);

            itemh[i]->setText(hstr2);
            config_page2->ui->tableWidget->setItem(i,0,itemh[i]);
            itemv[i]->setText(vstr2);
            config_page2->ui->tableWidget->setItem(i,1,itemv[i]);

            config_page2->Bar2[i]->setValue(100); //result
            QString name2 = hstr2+"_"+vstr2;
            config_page2->LineEd[i]->setText(name2); //file_name
        }
    }else if(dvo_flag == 2){
        //DVO0
        for(int i=0;i<cfg_grounp2;i++){ //2
            QString hstr1 = QString::number(parse_all[1].cfg[i]->hdisplay);
            QString vstr1 = QString::number(parse_all[1].cfg[i]->vdisplay);

            itemh2[i]->setText(hstr1);
            config_page1->ui->tableWidget->setItem(i,0,itemh2[i]);
            itemv2[i]->setText(vstr1);
            config_page1->ui->tableWidget->setItem(i,1,itemv2[i]);

            config_page1->Bar1[i]->setValue(100); //result
            QString name = hstr1+"_"+vstr1;
            config_page1->LineEd1[i]->setText(name); //file_name
        }
        //DVO1
        for(int j=0;j<cfg_grounp1;j++){
            QString hstr2 = QString::number(parse_all[0].cfg[j]->hdisplay);
            QString vstr2 = QString::number(parse_all[0].cfg[j]->vdisplay);

            itemh[j]->setText(hstr2);
            config_page2->ui->tableWidget->setItem(j,0,itemh[j]);
            itemv[j]->setText(vstr2);
            config_page2->ui->tableWidget->setItem(j,1,itemv[j]);

            config_page2->Bar2[j]->setValue(100); //result
            QString name2 = hstr2+"_"+vstr2;
            config_page2->LineEd[j]->setText(name2); //file_name
        }
    }
    return;
}


void Widget::parse_bin2ui()
{
    ui->tabWidget->setTabEnabled(0,true);
    ui->tabWidget->setTabEnabled(1,true);
    ui->dvo_all_btn->setChecked(true);
    this->load_encoder_to_ui();
    this->load_crtc_to_ui();
    this->load_connector_to_ui();
    this->load_pwm_to_ui();
    this->load_backlight_to_ui();
    this->load_i2c_to_ui();
    this->load_cfg_to_ui();
    return;
}

void Widget::free_parse_memory()
{
    delete [] parse_all->crtc;
    delete [] parse_all->encoder;
    delete [] parse_all->connector;
    delete [] parse_all->pwm;
    delete [] parse_all->bl;
    delete [] parse_all->i2c;
    for(int i=0;i<10;i++){
        delete [] parse_all[1].cfg[i];
        delete [] parse_all[0].cfg[i];
    }
    delete [] parse_all;
    return;
}

void allocate_parse_memory()
{
    parse_all = new all_data [2];
    if(!parse_all)
        qDebug("parse_all allocate err!\n");

    memset(parse_all,0,2*sizeof(struct all_data));
    for(int i=0;i<2;i++){
        parse_all[i].crtc = new vbios_crtc();
        memset(parse_all[i].crtc,0,sizeof(struct vbios_crtc));

        parse_all[i].encoder = new vbios_encoder();
        memset(parse_all[i].encoder,0,sizeof(struct vbios_encoder));

        parse_all[i].connector = new vbios_connector();
        memset(parse_all[i].connector,0,sizeof(struct vbios_connector));

        parse_all[i].pwm = new vbios_pwm();
        memset(parse_all[i].pwm,0,sizeof(struct vbios_pwm));

        parse_all[i].bl = new vbios_backlight();
        memset(parse_all[i].bl,0,sizeof(struct vbios_backlight));

        parse_all[i].i2c = new vbios_i2c();
        memset(parse_all[i].i2c,0,sizeof(struct vbios_i2c));

        for(int j=0;j<10;j++){
             parse_all[i].cfg[j] = new vbios_cfg_encoder();
             memset(parse_all[i].cfg[j],0,sizeof(vbios_cfg_encoder));
        }
    }
    return;
}

void Widget::clear_data_on_ui()
{
    this->dev_properlty_init();
    for(int i=0;i<2;i++){
         memset(parse_all[i].crtc,0,sizeof(struct vbios_crtc));
         memset(parse_all[i].encoder,0,sizeof(struct vbios_encoder));
         memset(parse_all[i].connector,0,sizeof(struct vbios_connector));
         memset(parse_all[i].pwm,0,sizeof(struct vbios_pwm));
         memset(parse_all[i].bl,0,sizeof(vbios_backlight));
         memset(parse_all[i].i2c,0,sizeof(struct vbios_i2c));

         for(int j=0;j<10;j++){
              memset(parse_all[i].cfg[j],0,sizeof(vbios_cfg_encoder));
         }
     }
    return;
}

void Widget::on_load_btn_clicked()
{
    this->clear_data_on_ui();
    QString name;
    name = QFileDialog::getOpenFileName(this,"打开",
                            QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation),"*(*.bin)");
    if(name.isEmpty())
         return;

    struct loongson_device *pldev;
    pldev = (struct loongson_device *)malloc(sizeof(struct loongson_device));
    pldev->vbios = (u8*)malloc(VBIOS_SIZE);

    parse_flag = 0;
    if(!pldev->vbios){
        printf("alloc vbios falling!\n");
    }
    memset(pldev->vbios,(u8)-1,VBIOS_SIZE);
    this->loongson_get_file(VBIOS_SIZE,pldev,name);
    this->loongson_vbios_init(pldev);
    this->parse_bin2ui();
    this->loongson_vbios_exit(pldev);
}

void Widget::on_faq_btn_clicked()
{
    QDesktopServices::openUrl(QUrl(QLatin1String("http://www.loongnix.org/index.php/%E9%A6%96%E9%A1%B5")));
}

void Widget::on_exit_btn_clicked()
{
    this->close();
}

void Widget::on_crtc_id_spin_2_valueChanged(int arg1)
{
    all[1].crtc->crtc_id = arg1;
}

void Widget::on_freq_spin_2_valueChanged(int arg1)
{
    all[1].crtc->max_freq = arg1;
}

void Widget::on_encoder_type_combo_2_currentIndexChanged(int index)
{
     all[1].encoder->type = (enum encoder_type)index;
}

void Widget::on_config_type_combo_2_currentIndexChanged(int index)
{
    if(index == 0)
        all[1].encoder->config_type = encoder_os_config;
    else if(index == 1)
        all[1].encoder->config_type = encoder_timing_filling;
    else if(index == 2)
        all[1].encoder->config_type = encoder_kernel_driver;

    if(all[1].encoder->config_type == encoder_timing_filling || all[1].encoder->config_type == encoder_kernel_driver )
        ui->encoder_config_btn_2->hide();
    else
        ui->encoder_config_btn_2->show();
}

void Widget::on_chip_combo_2_currentIndexChanged(int index)
{
     all[1].encoder->chip = (enum encoder_object)index;
}


void Widget::on_encoder_config_btn_2_clicked()
{
    config_page2->config_type = all[1].encoder->config_type;
    if(config_page2->config_type == encoder_os_config){
        config_page2->ui->tableWidget->setColumnHidden(0,false);
        config_page2->ui->tableWidget->setColumnHidden(1,false);
    }
    config_page2->show();
}


void Widget::on_connect_type_combo_2_currentIndexChanged(int index)
{
     all[1].connector->type = (enum connector_type)index;
}

void Widget::on_hotplug_combo_2_currentIndexChanged(int index)
{
    all[1].connector->hotplug = (enum hotplug)index;
    if(all[1].connector->hotplug == irq){
        ui->irq_plament_lab2->show();
        ui->irq_placement_combo_2->show();

        ui->irq_gpio_lab2->show();
        ui->irq_gpio_spin_2->show();
    }else{
        ui->irq_plament_lab2->hide();
        ui->irq_placement_combo_2->hide();

        ui->irq_gpio_lab2->hide();
        ui->irq_gpio_spin_2->hide();
    }
}

void Widget::on_edid_method_combo_currentIndexChanged(int index)
{
    if(index == 0){
        all[0].connector->edid_method = via_i2c;
    }else if(index == 1){
       all[0].connector->edid_method = via_vbios;
    }

    if(all[0].connector->edid_method == via_vbios)
        ui->connector_edid->show();
    else
        ui->connector_edid->hide();
}

void Widget::on_edid_method_combo_2_currentIndexChanged(int index)
{
     if(index == 0){
         all[1].connector->edid_method = via_i2c;
     }else if(index == 1){
        all[1].connector->edid_method = via_vbios;
     }

     if(all[1].connector->edid_method == via_vbios)
         ui->connector_edid_2->show();
     else
         ui->connector_edid_2->hide();
}

void Widget::on_pwm_id_spin_2_valueChanged(int arg1)
{
    all[1].pwm->pwm = arg1;
}

void Widget::on_polarity_spin_2_valueChanged(int arg1)
{
    all[1].pwm->polarity = arg1;
}

void Widget::on_peroid_spin_2_valueChanged(int arg1)
{
    all[1].pwm->peroid = arg1;
}


void Widget::on_connector_edid_2_clicked()
{
    edid_page2->show();
}

void Widget::on_irq_placement_combo_currentIndexChanged(int index)
{
    all[0].connector->irq_placement = (enum gpio_placement)index;
}

void Widget::on_irq_gpio_spin_valueChanged(int arg1)
{
    all[0].connector->irq_gpio = arg1;
}

void Widget::on_irq_placement_combo_2_currentIndexChanged(int index)
{
    all[1].connector->irq_placement = (enum gpio_placement)index;
}

void Widget::on_irq_gpio_spin_2_valueChanged(int arg1)
{
    all[1].connector->irq_gpio = arg1;
}

void Widget::on_document_btn_clicked()
{
//    QString project_path = QApplication::applicationFilePath();
//    QString str;
//    str = project_path.section("/",0,-3);

    QString pdf_path = "/usr/share/VBIOS.pdf";
//    QString full_path = str + "/" + pdf_path;
    QString filenName(pdf_path);

    if(filenName.isNull() || !filenName.contains(".pdf")) return;
    QDesktopServices::openUrl(QUrl::fromLocalFile(filenName));
}

void Widget::on_dvo0_btn_clicked()
{
    num = 1;
    ui->tabWidget->setTabEnabled(0,true);
    ui->tabWidget->setTabEnabled(1,false);
}

void Widget::on_dvo1_btn_clicked()
{
    num = 1;
    ui->tabWidget->setTabEnabled(0,false);
    ui->tabWidget->setTabEnabled(1,true);
}

void Widget::on_dvo_all_btn_clicked()
{
    num = 2;
    ui->tabWidget->setTabEnabled(0,true);
    ui->tabWidget->setTabEnabled(1,true);
    ui->tabWidget->setCurrentIndex(0);
}

void Widget::on_i2c_type_combo_currentIndexChanged(int index)
{
    all[0].i2c->type = (enum i2c_type)index;
}

void Widget::on_i2c_type_combo_2_currentIndexChanged(int index)
{
    all[1].i2c->type = (enum i2c_type)index;
}

void Widget::on_bl_type_combo_currentIndexChanged(int index)
{
    all[0].bl->type = (enum vbios_backlight_type)index;
}

void Widget::on_bl_used_spin_valueChanged(int arg1)
{
    all[0].bl->used = arg1;
}

void Widget::on_bl_type_combo2_currentIndexChanged(int index)
{
    all[1].bl->type = (enum vbios_backlight_type)index;
}

void Widget::on_bl_used_spin2_valueChanged(int arg1)
{
    all[1].bl->used = arg1;
}

