#ifndef WIDGET_H
#define WIDGET_H

#include <QWidget>
#include <QList>
#include "vbios.h"
#include "encoder_config_page1.h"
#include "ui_encoder_config_page1.h"
#include "encoder_config_page2.h"
#include "ui_encoder_config_page2.h"

#include "connector_edid_page1.h"
#include "connector_edid_page2.h"
#include "loongson_data.h"
#include "hexspinbox.h"
#include <QTableWidgetItem>

#define VBIOS_DESC_OFFSET 0x6000
#define VBIOS_SIZE 0x40000
#define VBIOS_PWM_ID 		0x0
#define VBIOS_PWM_PERIOD 	0x1
#define VBIOS_PWM_POLARITY 	0x2
#define ARRAY_SIZE(x)  (sizeof(x)/sizeof(x[0]))


void INIT_LIST_HEAD(struct list_head *list);
void __plist_add(struct list_head *newn, struct list_head * prev,struct list_head *next);
void plist_add(struct list_head *newn,struct list_head *phead);
void __list_del(struct list_head *prev,list_head *next);
void list_del(struct list_head *entry);
int print_func(struct loongson_device *ldev);

bool parse_vbios_default(struct desc_node *pthis,struct vbios_cmd *cmd);
parse_func get_parse_func(struct vbios_desc *vb_desc);
u32 insert_desc_list(struct loongson_device *ldev ,struct vbios_desc *vb_desc);

bool parse_vbios_backlight(struct desc_node *pthis, struct vbios_cmd *cmd);
bool parse_vbios_pwm(struct desc_node *pthis,struct vbios_cmd *cmd);
bool parse_vbios_header(struct desc_node *pthis,struct vbios_cmd *cmd);
bool parse_vbios_crtc(struct desc_node *pthis,struct vbios_cmd *cmd);
bool parse_vbios_i2c(struct desc_node *pthis,struct vbios_cmd *cmd);
bool parse_vbios_connector(struct desc_node *pthis,struct vbios_cmd *cmd);
bool parse_vbios_encoder(struct desc_node *pthis,struct vbios_cmd *cmd);
bool parse_vbios_cfg_encoder(struct desc_node *pthis,struct vbios_cmd *cmd);
void allocate_parse_memory();

namespace Ui {
class Widget;
}

struct vbios_crtc;
struct vbios_encoder;
struct vbios_connector;
struct vbios_pwm;
struct vbios_i2c;
struct vbios_cfg_encoder;
struct all_data;
struct loongson_device;
struct block_list;
class encoder_config_page1;
class encoder_config_page2;
class connector_edid_page1;
class connector_edid_page2;
struct edid_conn;
struct config_encoder;
struct cfg_info;
class hexspinbox;
class Widget : public QWidget
{
    Q_OBJECT
    friend class hexspinbox;
public:
    explicit Widget(QWidget *parent = nullptr);
    friend class encoder_config_page1;
    friend class encoder_config_page2;
    friend class connector_edid_page1;
    friend class connector_edid_page2;
    ~Widget();
    u32  get_data_size(void);
    u32  get_desc_node_num(void);
    void *get_data_by_type(enum desc_type type);
    int  fill_vbios_desc(void);
    int  fill_vbios_header(void);
    int  fill_vbios_data(u8 *mem_in);
    uint crc32(uint crc, char *buff, int len);
    void make_table();
    int  simu_vbios_header(int num_index);
    int  simu_vbios_crtc();
    int  simu_vbios_i2c();
    int  simu_vbios_encoder();
    int  simu_vbios_connector();
    int  simu_vbios_pwm();
    int  simu_vbios_backlight();
    int  simu_vbios_cfg_encoder();
    void saveVbios(const QString &filename);
    int  loongson_vbios_simu_data(s32 num_index);
    int  vbios_assembly_vbios();

public:
    void Set_shortcut_icon();
    void dev_properlty_init();
    void parse_bin2ui();
    void load_crtc_to_ui();
    void load_encoder_to_ui();
    void load_connector_to_ui();
    void load_i2c_to_ui();
    void load_pwm_to_ui();
    void load_backlight_to_ui();
    void load_cfg_to_ui();
    void free_parse_memory();
    void clear_data_on_ui();
    void clear_data_link1();
public:

    int  loongson_get_file(u32 size,struct loongson_device *ldev,QString fname);
    int  loongson_vbios_title_check(struct loongson_vbios *vbios);
    u32  parse_vbios_desc(struct loongson_device *ldev);
    int  loongson_vbios_init(struct loongson_device *ldev);
    int  loongson_vbios_exit(struct loongson_device *ldev);
    int  loongson_vbios_crc_check(void *vbios);
    void free_desc_list(loongson_device *ldev);
    void kernel_make_table(void);
    uint lscrc32(uint crc, unsigned char *buff, int len);

private slots:
    void on_tabWidget_currentChanged(int index);
    void on_crtc_id_spin_valueChanged(int arg1);
    void on_encoder_type_combo_currentIndexChanged(int index);
    void on_config_type_combo_currentIndexChanged(int index);
    void on_chip_combo_currentIndexChanged(int index);
    void on_connect_type_combo_currentIndexChanged(int index);
    void on_hotplug_combo_currentIndexChanged(int index);

    void on_pwm_id_spin_valueChanged(int arg1);
    void on_polarity_spin_valueChanged(int arg1);
    void on_peroid_spin_valueChanged(int arg1);
    void on_encoder_config_btn_clicked();
    void on_connector_edid_clicked();
    void on_save_btn_clicked();
    void on_check_btn_clicked();
    void on_load_btn_clicked();
    void on_faq_btn_clicked();
    void on_exit_btn_clicked();


    void on_crtc_id_spin_2_valueChanged(int arg1);
    void on_freq_spin_2_valueChanged(int arg1);
    void on_encoder_type_combo_2_currentIndexChanged(int index);
    void on_config_type_combo_2_currentIndexChanged(int index);

    void on_chip_combo_2_currentIndexChanged(int index);
    void on_encoder_config_btn_2_clicked();
    void on_connect_type_combo_2_currentIndexChanged(int index);
    void on_hotplug_combo_2_currentIndexChanged(int index);
    void on_edid_method_combo_2_currentIndexChanged(int index);
    void on_pwm_id_spin_2_valueChanged(int arg1);
    void on_polarity_spin_2_valueChanged(int arg1);
    void on_peroid_spin_2_valueChanged(int arg1);
    void on_connector_edid_2_clicked();

    void on_crtc_freq_spin_valueChanged(int arg1);
    void on_irq_placement_combo_currentIndexChanged(int index);
    void on_irq_gpio_spin_valueChanged(int arg1);
    void on_irq_placement_combo_2_currentIndexChanged(int index);
    void on_irq_gpio_spin_2_valueChanged(int arg1);

    void phy_edit_data();
    void phy_edit_data2();
    void on_document_btn_clicked();

    void on_dvo0_btn_clicked();

    void on_dvo1_btn_clicked();

    void on_dvo_all_btn_clicked();

    void on_i2c_type_combo_currentIndexChanged(int index);
    void on_i2c_type_combo_2_currentIndexChanged(int index);
    void on_edid_method_combo_currentIndexChanged(int index);

    void on_bl_type_combo_currentIndexChanged(int index);

    void on_bl_used_spin_valueChanged(int arg1);

    void on_bl_type_combo2_currentIndexChanged(int index);

    void on_bl_used_spin2_valueChanged(int arg1);

private:
    uint POLYNOMIAL = 0xEDB88320 ;
    int have_table = 0 ;
    uint table[256];

public:
    Ui::Widget *ui;
    all_data *all;
    encoder_config_page1 *config_page1;
    encoder_config_page2 *config_page2;

    connector_edid_page1 *edid_page1;
    connector_edid_page2 *edid_page2;
    HexSpinBox *hexspin1;
    HexSpinBox *hexspin2;

    QList<QTableWidgetItem *> itemh;
    QList<QTableWidgetItem *> itemv;

    QList<QTableWidgetItem *> itemh2;
    QList<QTableWidgetItem *> itemv2;

    QList<QPushButton *> bt2;

    QLinkedList<struct block_list> vbt_block_list;
    QLinkedList<struct block_list>::iterator bliterator;
    QLinkedList<struct loongson_device> ldev;
    QLinkedList<struct loongson_device>::iterator ldevit;
    loongson_device *mem_node;
};

#endif // WIDGET_H

